# -*- coding: utf-8 -*-

import json
import urllib2
from contextlib import closing
from urllib import urlencode
from utils import get_ip_address

__all__ = [
    "GoogleTranslator",
]

class GoogleTranslator(object):
    """
    Translator with Google Translate API with version 2
    http://code.google.com/intl/ja/apis/language/translate/overview.html
    http://code.google.com/intl/ja/apis/language/translate/v2/using_rest.html
    see also
    Google Translate API Terms of Use
    http://code.google.com/intl/ja/apis/language/translate/terms.html
    """
    domain = "www.googleapis.com"
    common_path = "/language/translate/"
    apikey = "AIzaSyDDCHHwbfHLIsHWEhxAu41UmrRCg_Xmvm8"
    q_format = "html"

    def __init__(self, lang_from, lang_to, handler):
        self.lang_from = lang_from
        self.lang_to = lang_to
        self.handler = handler
        self._url = "https://{0}{1}".format(self.domain, self.common_path)

    def get_url(self, func=None):
        if func:
            url = "{0}v2/{1}".format(self._url, func.func_name)
        else:
            url = "{0}v2".format(self._url)
        return url

    def request(self, url, data):
        req = urllib2.Request(url)
        req.add_header("X-HTTP-Method-Override", "GET")
        req.add_data(data)
        with closing(urllib2.urlopen(req)) as res:
            res_json = json.loads(res.read())
        return res_json["data"]

    def call_api(self, query, key, func=None):
        """high-level method to make url and request"""
        url = self.get_url(func)
        response = self.request(url, urlencode(query, doseq=True))
        return self.api(), response[key]

    def detect(self, texts):
        """ Detect Language
        http://code.google.com/intl/ja/apis/language/translate/v2/\
        using_rest.html#detect-language
        """
        query = {
            "key": self.apikey,
            "q": [t.encode("utf-8") for t in texts],
        }
        return self.call_api(query, u"detections", func=self.detect)

    def languages(self):
        """ Discover Supported Languages
        http://code.google.com/intl/ja/apis/language/translate/v2/\
        using_rest.html#supported-languages
        """
        query = {
            "key": self.apikey,
            "target": self.lang_to,
        }
        return self.call_api(query, u"languages", func=self.languages)

    def translate(self, texts):
        """ Translate Text
        http://code.google.com/intl/ja/apis/language/translate/v2/\
        using_rest.html#Translate
        """
        query = {
            "format": self.q_format,
            "key": self.apikey,
            "q": [t.encode("utf-8") for t in texts],
            "source": self.lang_from,
            "target": self.lang_to,
        }
        api, response = self.call_api(query, u"translations")
        return api, [self.parse_html(r[u"translatedText"]) for r in response]

    def translate_tts(self, text, lang, f):
        """ Unofficial Google Text To Speech API
        http://weston.ruter.net/projects/google-tts/
        """
        headers = {"User-Agent":
            "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-US) "
            "AppleWebKit/534.16 (KHTML, like Gecko) "
            "Chrome/10.0.648.204 Safari/534.16"
        }
        query = {"tl": lang, "q": text.encode("utf-8")}
        url = "http://translate.google.com/translate_tts?{0}".format(
                urlencode(query))
        req = urllib2.Request(url, None, headers)
        with closing(urllib2.urlopen(req)) as res:
            f.write(res.read())
            f.flush()
        return self.api()

class GoogleTranslatorV1(object):
    """
    Translator with Google Translate API
    http://code.google.com/intl/ja/apis/language/translate/overview.html
    see also
    Google Translate API Terms of Use
    http://code.google.com/intl/ja/apis/language/translate/terms.html
    """
    domain = "ajax.googleapis.com"
    common_path = "/ajax/services/language/"
    apikey = "ABQIAAAAK6kpHnylgmAYtO7ZX01XXRSvW2ISZ2KI4wU-F"\
             "k6WlRk77d73EhTtYeI1LUl3BfkKv-17KKEWzdRTMw"
    userip = get_ip_address()
    q_format = "html"

    def __init__(self, lang_from, lang_to, handler):
        self.lang_from = lang_from
        self.lang_to = lang_to
        self.handler = handler
        self._url = "http://{0}{1}".format(self.domain, self.common_path)

    def get_api_path(self, func):
        return "{0}?".format(func.func_name)

    def get_url(self, func, query):
        return "{0}{1}{2}".format(self._url,
                self.get_api_path(func), urlencode(query))

    def request(self, url):
        req = urllib2.Request(url)
        with closing(urllib2.urlopen(req)) as res:
            res_json = json.loads(res.read())
            if res_json["responseStatus"] == 200:
                response = res_json["responseData"]
            else:
                raise RuntimeError(res_json)
        return response

    def call_api(self, func, query, key=None):
        """high-level method to make url and request"""
        url = self.get_url(func, query)
        response = self.request(url)
        if key:
            response = response[key]
        return self.api(), response

    def detect(self, text):
        """ JSON Developer's Guide for Language Detect
        http://code.google.com/intl/ja/apis/language/translate/v1/\
        using_rest_langdetect.html
        """
        query = {
            "v": "1.0",
            "q": text.encode("utf-8"),
            "key": self.apikey,
            "userip": self.userip,
        }
        return self.call_api(self.detect, query, "language")

    def translate(self, text):
        """ JSON Developer's Guide for Translate
        http://code.google.com/intl/ja/apis/language/translate/v1/\
        using_rest_translate.html
        """
        query = {
            "v": "1.0",
            "q": text.encode("utf-8"),
            "langpair": "{0}|{1}".format(self.lang_from, self.lang_to),
            "key": self.apikey,
            "userip": self.userip,
            "format": self.q_format,
        }
        _key = "translatedText"
        api, response = self.call_api(self.translate, query, _key)
        return api, self.parse_html(response)



# -*- coding: utf-8 -*-

import json
import urllib2
from contextlib import closing
from urllib import urlencode

__all__ = [
    "YahooTranslator",
]

class YahooTranslator(object):
    """
    Translator with Yahoo! Pipes
    http://pipes.yahoo.com/pipes/
    see also
    Yahoo! Pipes Terms of Use
    http://info.yahoo.com/legal/us/yahoo/pipes/pipes-4396.html
    """
    domain = "pipes.yahoo.com"
    common_path = "/t2y1979/"

    def __init__(self, lang_from, lang_to, handler):
        self.lang_from = lang_from
        self.lang_to = lang_to
        self.handler = handler
        self._url = "http://{0}{1}".format(self.domain, self.common_path)
        self.query = {"_render": "json"}

    def _get_api(self, lang_from, lang_to):
        # convert for for arbitrary api name
        api = "{0}2{1}?".format(lang_from, lang_to)
        if lang_from == "ja" and lang_to == "en":
            api = "ja2en_?"
        elif lang_from == "en" and lang_to.lower() == "zh-cn":
            api = "en2zhcn?"
        elif lang_from == "en" and lang_to.lower() == "zh-tw":
            api = "en2zhtw?"
        return api

    def get_api_path(self, func):
        if func.func_name == "translate":
            return self._get_api(self.lang_from, self.lang_to)
        elif func.func_name == "detect":
            return "{0}?".format(func.func_name)

    def get_url(self, func, query):
        return "{0}{1}{2}".format(self._url,
                self.get_api_path(func), urlencode(query))

    def request(self, url):
        req = urllib2.Request(url)
        with closing(urllib2.urlopen(req)) as res:
            res_json = json.loads(res.read())
        return res_json

    def call_api(self, func, query):
        """high-level method to make url and request"""
        url = self.get_url(func, query)
        response = self.request(url)
        return self.api(), response

    def detect(self, text):
        return self.api(), u"Not Supported"

    def translate(self, text):
        self.query.update(text=text.encode("utf-8"))
        api, response = self.call_api(self.translate, self.query)
        return api, response["value"]["items"][0]["description"]



# -*- coding: utf-8 -*-

from nose.tools import *

# functions for test
from ikazuchi.core.translator import TranslatingYahoo

class TestYahooTranslator(object):

    def setup(self):
        self.t = TranslatingYahoo("ja", "en", None)

    def test_get_api_path(self):
        def translate(): pass

        assert_equal("ja2en_?", self.t.get_api_path(translate))

    def test_detect(self):
        res = self.t.detect(unicode("テスト", "utf-8"))
        assert_equal(u"Not Supported", res[1])

    def test_translate(self):
        assert_equal(('Yahoo', u'Test'),
                     self.t.translate(unicode("テスト", "utf-8")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import sys

from conf import (get_conf, get_conf_path)
from core.translator import TRANSLATE_API
from locale import _
from utils import *

__version__ = "0.4.1"

def get_args():
    parser = argparse.ArgumentParser()
    parser.set_defaults(api="google", detect=False, lang=False,
                        lang_from="en", lang_to=get_lang(),
                        po_file=None, rst_file=None, sentence=None,
                        encoding=None, quiet=False, verbose=False)
    parser.add_argument("-a", "--api", dest="api", metavar="API",
                        help=u"APIs are {0}".format(TRANSLATE_API.keys()))
    parser.add_argument("-d", "--detect", dest="detect", action="store_true",
                        help=u"detect language for target sentence")
    parser.add_argument("-e", "--encoding", dest="encoding",
                        action=EncodingAction, metavar="ENCODING",
                        help=u"input/output encoding")
    parser.add_argument("-f", "--from", dest="lang_from", metavar="LANG",
                        help=u"original language")
    parser.add_argument("-l", "--languages", dest="lang", action="store_true",
                        help=u"show supported languages")
    parser.add_argument("-p", "--pofile", dest="po_file", nargs=1,
                        metavar="POFILE", help=u"target po file")
    parser.add_argument("-q", "--quiet", dest="quiet", action="store_true",
                        help=u"not to show original sentence to stdout")
    parser.add_argument("-r", "--rstfile", dest="rst_file", nargs=1,
                        metavar="RSTFILE", help=u"target reST file")
    parser.add_argument("-s", "--sentences", dest="sentences", nargs="+",
                        metavar="SENTENCE", help=u"target sentences")
    parser.add_argument("-t", "--to", dest="lang_to", metavar="LANG",
                        help=u"target language to translate")
    parser.add_argument("--version", action="version",
                        version="%(prog)s {0}".format(__version__))

    opts = parser.parse_args()
    if not opts.lang_to:
        opts.lang_to = raw_input(u"Type language code: ")

    err_msg = None
    if opts.api not in TRANSLATE_API.keys():
        err_msg = _(u"Unsupported API: {0}").format(opts.api)
    elif opts.po_file and not os.access(opts.po_file[0], os.R_OK):
        err_msg = _(u"Cannot access po file: {0}").format(opts.po_file[0])
    elif opts.rst_file and not os.access(opts.rst_file[0], os.R_OK):
        err_msg = _(u"Cannot access reST file: {0}").format(opts.rst_file[0])
    elif opts.encoding:
        err_encoding = check_encoding(opts.encoding)
        if err_encoding:
            err_msg = _(u"Unknown encodings: {0}").format(err_encoding)
    elif not (opts.lang or opts.po_file or opts.rst_file or opts.sentences):
        err_msg = _(u"Need to specify optional arguments")

    if err_msg:
        parser.print_help()
        print err_msg
        sys.exit(0)

    if not opts.encoding:
        set_default_encoding(opts)
    convert_str_to_unicode(opts)
    return opts

def main():
    # pre process
    check_python_version()
    conf = get_conf(get_conf_path())
    # main process
    opts = get_args()
    handler = get_handler(opts)
    t = TRANSLATE_API[opts.api](opts.lang_from, opts.lang_to, handler)
    t.set_apikey_from_conf(conf)
    t.call_method_with_handler()

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import sys
import tempfile
from nose.tools import *
from StringIO import StringIO

# functions for test
from ikazuchi.core.handler import *


class TestPOFileHandler(object):

    def setup(self):
        def make_po_file(path):
            f = tempfile.NamedTemporaryFile(dir=path)
            f.write('msgid "forest"\nmsgstr ""\nmsgid "book"\nmsgstr "hon"')
            f.flush()
            return f
        self.po_file = make_po_file(tempfile.tempdir)

    def teardown(self):
        self.po_file.close()

    def test_select_translation(self):
        data = [
            (["ref", "", "y"], "ref"),
            (["ref", "cur", "y"], "ref"),
            (["ref", "cur", ""], "cur"),
            (["ref", "", ""], ""),
            (["ref", "cur", "ent"], "ent"),
            (["ref", "", "ent"], "ent"),
        ]
        h = POFileHandler("google", self.po_file.name, ["utf-8", "utf-8"])
        for d in data:
            assert_equals(d[1], h._select_translation(*d[0]))


class TestSingleSentenceHandler(object):

    class Option(object):
        api = "google"
        sentences = [""]
        encoding = ["utf-8", "utf-8"]
        quiet = False
        detect = False

    def setup(self):
        sys.stdout = StringIO()
        self.opts = TestSingleSentenceHandler.Option()

    def _dummy_translate(self, sentence):
        return "", sentence

    def test_with_quiet_option(self):
        self.opts.quiet = True
        h = SingleSentenceHandler(self.opts)
        h._call_method(self._dummy_translate)
        first_line = sys.stdout.getvalue().split('\n')[0]
        assert_equal("translate():             ", first_line)

    def test_detect_with_quiet_option(self):
        self.opts.quiet = True
        self.opts.detect = True
        h = SingleSentenceHandler(self.opts)
        h._call_method(self._dummy_translate)
        first_line = sys.stdout.getvalue().split('\n')[0]
        assert_equal("detect():                ", first_line)

    def test_without_quiet_option(self):
        h = SingleSentenceHandler(self.opts)
        h._call_method(self._dummy_translate)
        first_line = sys.stdout.getvalue().split('\n')[0]
        assert_equal("sentence:                ", first_line)



# -*- coding: utf-8 -*-

DATA_SET = [
    (
        u"msgid",
        u"msgid"
    ),
    (
        u"msgid: %s",
        u"msgid: <span class=notranslate>%s</span>"
    ),
    (
        u"msgid: %d",
        u"msgid: <span class=notranslate>%d</span>"
    ),
    (
        u"msgid: %r",
        u"msgid: <span class=notranslate>%r</span>"
    ),
    (
        u"msgid: %(name)s",
        u"msgid: <span class=notranslate>%(name)s</span>"
    ),
    (
        u"msgid: %(name)d",
        u"msgid: <span class=notranslate>%(name)d</span>"
    ),
    (
        u"msgid: %(name)r",
        u"msgid: <span class=notranslate>%(name)r</span>"
    ),
    (
        u"msgid: '%s'",
        u"msgid: <span class=notranslate>'%s'</span>"
    ),
    (
        u"msgid: '%d'",
        u"msgid: <span class=notranslate>'%d'</span>"
    ),
    (
        u"msgid: '%r'",
        u"msgid: <span class=notranslate>'%r'</span>"
    ),
    (
        u'msgid: "%s"',
        u'msgid: <span class=notranslate>"%s"</span>'
    ),
    (
        u'msgid: "%d"',
        u'msgid: <span class=notranslate>"%d"</span>'
    ),
    (
        u'msgid: "%r"',
        u'msgid: <span class=notranslate>"%r"</span>'
    ),
]



# -*- coding: utf-8 -*-

import polib
import re
from base import BaseHandler

try:
    from ikazuchi.locale import _
except ImportError:
    def _(s): return s

_NOTRANSLATE_PTRN = re.compile(r"""(
      ["|']*(%|%(.+?))[d|r|s]["|']*
)""", re.U | re.X)

class POFileHandler(BaseHandler):
    """
    Handler class for translating PO file interactively
    """
    max_query = 100

    def __init__(self, api, po_file, encoding):
        if api == "microsoft":
            self.method_name = "translate_array"
        self.encoding = encoding
        self.po = polib.pofile(po_file, autodetect_encoding=False,
                               encoding=self.encoding[1])
        self.po.metadata["Content-Type"] = "text/plain; charset={0}".format(
                                                self.encoding[1])

    def _select_translation(self, ref, current, entered):
        """define which translated string use"""
        s = entered
        if entered.lower() == "y":
            s = ref
        elif current and entered == "":
            s = current
        return s

    def _call_method(self, api_method):
        """translate msgid in po file"""
        _prompt = _(u"Input: ").encode(self.encoding[0])
        for pos in range(0, len(self.po), self.max_query):
            sliced_po = self.po[pos:pos + self.max_query]
            items = [self.markup_msgid_notranslate(p.msgid) for p in sliced_po]
            api, ref = api_method(items)
            _reference = _(u"reference({0}):").format(api)
            for num, p in enumerate(sliced_po):
                _ref = ref[num]
                print u"{0:25}{1}".format(_(u"msgid:"), p.msgid)
                if p.msgstr:
                    print u"{0:25}{1}".format(_(u"current msgstr:"), p.msgstr)
                print u"{0:25}{1}".format(_reference, _ref)
                entered = unicode(raw_input(_prompt), self.encoding[0])
                p.msgstr = self._select_translation(_ref, p.msgstr, entered)
                self.po.save()
                print u"{0:25}{1}".format(_(u"updated msgstr:"), p.msgstr)
                print ""

    @classmethod
    def markup_msgid_notranslate(self, msgid):
        msgid = msgid.replace(u"&", u"&amp;")
        msgid = msgid.replace(u"<", u"&lt;")
        msgid = msgid.replace(u">", u"&gt;")
        repl = r"<span class=notranslate>\1</span>"
        return re.sub(_NOTRANSLATE_PTRN, repl, msgid)



# -*- coding: utf-8 -*-

from nose.tools import *

# functions for test
from ikazuchi.core.handler.pofile import *

def test_markup_msgid_notranslate():
    from data.po.markup_notranslate import DATA_SET
    _fmt = u"{0}_{1} ({2} ...)"
    _func = u"test_markup_msgid_notranslate"
    for num, (data, expected) in enumerate(DATA_SET):
        name = _fmt.format(_func, num, str(data)[:10])
        _assert = lambda e, a: assert_equal(e, a)
        _assert.description = name
        yield _assert, expected, POFileHandler.markup_msgid_notranslate(data)



# -*- coding: utf-8 -*-

from audio import AudioHandler
from base import LanguageHandler
from pofile import POFileHandler
from rstfile import reSTFileHandler
from text import SingleSentenceHandler

__all__ = [
    "AudioHandler",
    "LanguageHandler",
    "POFileHandler",
    "reSTFileHandler",
    "SingleSentenceHandler",
]



# -*- coding: utf-8 -*-

import sys
from setuptools import setup, find_packages
from os.path import join as pathjoin

VERSION = "0.4.1"
LONG_DESCRIPTION = "".join([
    open(pathjoin("src","README.txt")).read(),
    open(pathjoin("src","TODO.txt")).read()])

CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Environment :: Web Environment",
    "Intended Audience :: Developers",
    "Intended Audience :: System Administrators",
    "License :: OSI Approved :: Apache Software License",
    "Operating System :: MacOS :: MacOS X",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: POSIX",
    "Programming Language :: Python",
    "Topic :: Software Development",
    "Topic :: Software Development :: Internationalization",
]

INSTALL_REQUIRES = ["setuptools", "polib"]
if sys.version_info < (2, 7):
    INSTALL_REQUIRES.append("argparse")

setup(
     name="ikazuchi",
     version=VERSION,
     description="ikazuchi helps to translate document using web translate APIs efficiently",
     long_description=LONG_DESCRIPTION,
     classifiers=CLASSIFIERS,
     keywords=["po-file", "translate", "i18n", "internationalization"],
     author="Tetsuya Morimoto",
     author_email="tetsuya dot morimoto at gmail dot com",
     url="http://t2y.bitbucket.org/ikazuchi/build/html/index.html",
     license="Apache License 2.0",
     py_modules=["raimei", "enrai"],
     packages=find_packages("src"),
     package_dir={"": "src"},
     package_data = {"": ["buildout.cfg"]},
     include_package_data=True,
     install_requires=INSTALL_REQUIRES,
     extras_require=dict(
         test=[
             "Nose",
             "minimock",
             "pep8",
         ],
     ),
     test_suite="nose.collector",
     tests_require=["Nose","minimock","pep8"],
     entry_points="""
        [console_scripts]
        ikazuchi = ikazuchi:main
        raimei = raimei:main
        enrai = enrai:main
     """,
)



# -*- coding: utf-8 -*-

from os.path import (dirname, realpath, join as pathjoin)

BASE_DIR = dirname(realpath(__file__))
CORE_DIR = pathjoin(BASE_DIR, "core")
LOCALE_DIR = pathjoin(BASE_DIR, "locale")

from ikazuchi import main



# -*- coding: utf-8 -*-

from base import BaseHandler
from ikazuchi.core.translator import TRANSLATE_API as API
from tempfile import NamedTemporaryFile

class AudioHandler(BaseHandler):
    """
    Handler class for text-to-speech
    """
    def __init__(self, opts):
        self.is_translate = False
        if opts.sentences[0] == "audio":
            self.sentences = opts.sentences[1:]
            self.lang = opts.lang_from
        else:
            self.sentences = opts.sentences[:-1]
            self.lang = opts.lang_to
            self.is_translate = True
        self.api = opts.api
        self.encoding = opts.encoding
        self.quiet = opts.quiet
        self.translator = API[opts.api](opts.lang_from, opts.lang_to, None)
        if self.api == "google":
            self.method_name = "translate_tts"
        elif self.api == "microsoft":
            self.method_name = "speak"

    def _encode(self, text):
        return text.encode(self.encoding[1])

    def _translate(self, texts):
        if self.api == "google":
            api, translated = self.translator.translate(texts)
        elif self.api == "microsoft":
            api, translated = self.translator.translate_array(texts)
        return translated

    def _call_method(self, api_method):
        texts = self.sentences
        if self.is_translate:
            texts = self._translate(self.sentences)
        _trans = u"{0}({1}):".format("translate", self.api.title())
        for num, text in enumerate(texts):
            if not self.quiet:
                print self._encode(u"{0:25}{1}".format(
                            "sentence:", self.sentences[num]))
            if self.is_translate:
                print self._encode(u"{0:25}{1}".format(_trans, text))
            with NamedTemporaryFile(mode="wb") as tmp:
                api = api_method(text, self.lang, tmp)
                _method = u"{0}({1}):".format(self.method_name, api)
                print self._encode(u"{0:25}".format(_method))
                self.play_audio(tmp.name)

    def play_audio(self, file_name):
        import platform
        os_name = platform.system()
        if os_name == "Darwin":
            import subprocess
            subprocess.call(["afplay", file_name])
        elif os_name in ("Linux", "FreeBSD"):
            self.play_with_ossaudiodev(file_name)
        else:
            print "Not supported for playing audio: {0}".format(os_name)

    def play_with_ossaudiodev(self, file_name):
        import sys
        import sndhdr
        from contextlib import closing, nested
        from ossaudiodev import open as oss_open
        from wave import open as wave_open
        file_info = sndhdr.what(file_name)
        if not file_info or file_info[0] != "wav":
            print "Not supported audio file type"
            return
        with nested(closing(wave_open(file_name, "rb")),
                    closing(oss_open("w"))) as (wav, dev):
            nc, sw, fr, nf, comptype, compname = wav.getparams()
            try:
                from ossaudiodev import (AFMT_S16_NE, AFMT_S16_BE, AFMT_S16_LE)
            except ImportError:
                AFMT_S16_NE = AFMT_S16_BE
                if sys.byteorder == "little":
                    AFMT_S16_NE = AFMT_S16_LE
            dev.setparameters(AFMT_S16_NE, nc, fr)
            data = wav.readframes(nf)
            dev.write(data)
